@@ -22,35 +22,14 @@ Crypt::Perl::ECDSA::PrivateKey - object representation of ECDSA private key
     #$payload is probably a hash (e.g., SHA-256) of your original message.
     my $sig = $prkey->sign($payload);
 
-    #For JSON Web Algorithms (JWT et al.), cf. RFC 7518 page 8
-    #This will also apply the appropriate SHA algorithm before signing.
-    my $sig_jwa = $prkey->sign_jwa($payload);
-
     $prkey->verify($payload, $sig) or die "Invalid signature!";
-    $prkey->verify_jwa($payload, $sig_jwa) or die "Invalid signature!";
 
     #Corresponding “der” methods exist as well.
     my $cn_pem = $prkey->to_pem_with_curve_name();
     my $expc_pem = $prkey->to_pem_with_explicit_curve();
 
-    #----------------------------------------------------------------------
-
     my $pbkey = $prkey->get_public_key();
 
-    #----------------------------------------------------------------------
-
-    #Includes “kty”, “crv”, “x”, “y”, and (for private) “d”.
-    #Add in whatever else your application needs afterward.
-    #
-    #These will die() if you try to run it with a curve that
-    #doesn’t have a known JWK “crv” value.
-    #
-    my $prv_jwk = $prkey->get_struct_for_private_jwk();
-    my $pub_jwk = $prkey->get_struct_for_public_jwk();
-
-    #Useful for JWTs
-    my $jwt_alg = $pbkey->get_jwa_alg();
-
 =head1 DISCUSSION
 
 The SYNOPSIS above should be illustration enough of how to use this class.
@@ -81,9 +60,12 @@ use parent qw( Crypt::Perl::ECDSA::KeyBase );
 use Try::Tiny;
 
 use Bytes::Random::Secure::Tiny ();
+use Module::Load ();
 
 use Crypt::Perl::ASN1 ();
 use Crypt::Perl::BigInt ();
+use Crypt::Perl::PKCS8 ();
+use Crypt::Perl::RNG ();
 use Crypt::Perl::Math ();
 use Crypt::Perl::ToDER ();
 use Crypt::Perl::X ();
@@ -103,7 +85,28 @@ use constant ASN1_PRIVATE => Crypt::Perl::ECDSA::KeyBase->ASN1_Params() . q<
 
 use constant _PEM_HEADER => 'EC PRIVATE KEY';
 
-use constant NUMBER_CLASS => 'Crypt::Perl::BigInt';
+#Expects $key_parts to be a hash ref:
+#
+#   version - AFAICT unused
+#   private - BigInt or its byte-string representation
+#   public  - ^^
+#
+sub new_by_curve_name {
+    my ($class, $key_parts, $curve_name) = @_;
+
+    #We could store the curve name on here if looking it up
+    #in to_der_with_curve_name() proves prohibitive.
+    return $class->new(
+        $key_parts,
+
+        #“Fake out” the $curve_parts attribute by recreating
+        #the structure that ASN.1 would give from a named curve.
+        {
+            namedCurve => Crypt::Perl::ECDSA::EC::DB::get_oid_for_curve_name($curve_name),
+        },
+    );
+}
+
 
 #$curve_parts is also a hash ref, defined as whatever the ASN.1
 #parse of the main key’s “parameters” returned, whether that be
@@ -112,132 +115,27 @@ use constant NUMBER_CLASS => 'Crypt::Perl::BigInt';
 sub new {
     my ($class, $key_parts, $curve_parts) = @_;
 
-    if (!length $key_parts->{'version'}) {
-        die Crypt::Perl::X::create('Generic', 'Need a “version”! (Try 1)');
-    }
-
     my $self = {
         version => $key_parts->{'version'},
     };
 
-    bless $self, $class;
-
-    $self->_set_public( $key_parts->{'public'} );
-
-    for my $k ( qw( private ) ) {
-        if ( try { $key_parts->{$k}->isa(NUMBER_CLASS()) } ) {
+    for my $k ( qw( private public ) ) {
+        if ( try { $key_parts->{$k}->isa('Crypt::Perl::BigInt') } ) {
             $self->{$k} = $key_parts->{$k};
         }
         else {
-            die Crypt::Perl::X::create('Generic', sprintf "“$k” must be “%s”, not “$key_parts->{$k}”!", NUMBER_CLASS());
+            die "“$k” must be “Crypt::Perl::BigInt”, not “$key_parts->{$k}”!";
         }
     }
 
+    bless $self, $class;
+
     return $self->_add_params( $curve_parts );
 }
 
-sub sign {
-    return $_[0]->_sign_and_serialize($_[1]);
-}
-
-sub _sign_and_serialize {
-    my ($self, $whatsit, $hashfn) = @_;
-
-    my ($r, $s) = $self->_sign($whatsit, $hashfn);
-    return $self->_serialize_sig( $r, $s );
-}
-
-sub _hash_sign_and_serialize {
-    my ($self, $whatsit, $hashfn) = @_;
-
-    require Digest::SHA;
-    $whatsit = Digest::SHA->can($hashfn)->($whatsit);
-
-    return $self->_sign_and_serialize($whatsit, $hashfn);
-}
-
-sub sign_sha1 {
-    my ($self, $whatsit) = @_;
-
-    return $_[0]->_hash_sign_and_serialize($whatsit, 'sha1');
-}
-
-sub sign_sha224 {
-    my ($self, $whatsit) = @_;
-
-    return $_[0]->_hash_sign_and_serialize($whatsit, 'sha224');
-}
-
-sub sign_sha256 {
-    my ($self, $whatsit) = @_;
-
-    return $_[0]->_hash_sign_and_serialize($whatsit, 'sha256');
-}
-
-sub sign_sha384 {
-    my ($self, $whatsit) = @_;
-
-    return $_[0]->_hash_sign_and_serialize($whatsit, 'sha384');
-}
-
-sub sign_sha512 {
-    my ($self, $whatsit) = @_;
-
-    return $_[0]->_hash_sign_and_serialize($whatsit, 'sha512');
-}
-
-#cf. RFC 7518, page 8
-sub sign_jwa {
-    my ($self, $whatsit) = @_;
-
-    # As of version 0.34 this method creates deterministic signatures.
-
-    my $dgst_name = $self->_get_jwk_digest_name();
-
-    require Digest::SHA;
-    $whatsit = Digest::SHA->can($dgst_name)->($whatsit);
-
-    my ($r, $s) = map { $_->as_bytes() } $self->_sign($whatsit, $dgst_name);
-
-    my $octet_length = Crypt::Perl::Math::ceil($self->max_sign_bits() / 8);
-
-    substr( $_, 0, 0 ) = "\0" x ($octet_length - length) for ($r, $s);
-
-    return $r . $s;
-}
-
-sub get_public_key {
-    my ($self) = @_;
-
-    require Crypt::Perl::ECDSA::PublicKey;
-
-    my $curve_hr = $self->_explicit_curve_parameters( seed => 1 );
-    my $ccurve_hr = $curve_hr->{'ecParameters'}{'curve'};
-    $ccurve_hr->{'seed'} = [ $ccurve_hr->{'seed'} ];
-
-    return Crypt::Perl::ECDSA::PublicKey->new(
-        $self->_decompress_public_point(),
-        $curve_hr,
-    );
-}
-
-sub get_struct_for_private_jwk {
-    my ($self) = @_;
-
-    my $hr = $self->get_struct_for_public_jwk();
-
-    require MIME::Base64;
-
-    $hr->{'d'} = MIME::Base64::encode_base64url( $self->{'private'}->as_bytes() );
-
-    return $hr;
-}
-
-#----------------------------------------------------------------------
-
 #$whatsit is probably a message digest, e.g., from SHA256
-sub _sign {
-    my ($self, $whatsit, $det_hashfuncname) = @_;
+sub sign {
+    my ($self, $whatsit) = @_;
 
     my $dgst = Crypt::Perl::BigInt->from_bytes( $whatsit );
 
@@ -253,39 +151,53 @@ sub _sign {
 
     #isa ECPoint
     my $G = $self->_G();
+#printf "G.x: %s\n", $G->{'x'}->to_bigint()->as_hex();
+#printf "G.y: %s\n", $G->{'y'}->to_bigint()->as_hex();
+#printf "G.z: %s\n", $G->{'z'}->as_hex();
+
     my ($k, $r);
 
     do {
-        if ($det_hashfuncname) {
-            require Crypt::Perl::ECDSA::Deterministic;
-            $k = Crypt::Perl::ECDSA::Deterministic::generate_k(
-                $n,
-                $priv_num,
-                $whatsit,
-                $det_hashfuncname,
-            );
-        }
-        else {
-            $k = Crypt::Perl::Math::randint($n);
-        }
-
+        $k = Crypt::Perl::Math::randint($n);
+#print "once\n";
+#printf "big random: %s\n", $k->as_hex();
+#$k = Crypt::Perl::BigInt->new("98452900523450592996995215574085435893040452563985855319633891614520662229711");
+#printf "k: %s\n", $k->bstr();
         my $Q = $G->multiply($k);   #$Q isa ECPoint
+#printf "Q.x: %s\n", $Q->{'x'}->to_bigint()->as_hex();
+#printf "Q.y: %s\n", $Q->{'y'}->to_bigint()->as_hex();
+#printf "Q.z: %s\n", $Q->{'z'}->as_hex();
+        $r = $Q->get_x()->to_bigint()->bmod($n);
+    } while ($r <= 0);
 
-        $r = $Q->get_x()->to_bigint()->copy()->bmod($n);
-    } while !$r->is_positive();
+#printf "k: %s\n", $k->as_hex();
+#printf "n: %s\n", $n->as_hex();
+#printf "e: %s\n", $dgst->as_hex();
+#printf "d: %s\n", $priv_num->as_hex();
+#printf "r: %s\n", $r->as_hex();
 
     my $s = $k->bmodinv($n);
+    $s *= ( $dgst + ( $priv_num * $r ) );
+    $s %= $n;
 
-    #$s *= ( $dgst + ( $priv_num * $r ) );
-    $s->bmul( $priv_num->copy()->bmuladd( $r, $dgst ) );
-
-    $s->bmod($n);
-
-    return ($r, $s);
+    return $self->_serialize_sig( $r, $s );
 }
 
+sub get_public_key {
+    my ($self) = @_;
+
+    Module::Load::load('Crypt::Perl::ECDSA::PublicKey');
+
+    return Crypt::Perl::ECDSA::PublicKey->new(
+        $self->{'public'},
+        $self->_explicit_curve_parameters(),
+    );
+}
+
+#----------------------------------------------------------------------
+
 sub _get_asn1_parts {
-    my ($self, $curve_parts, @params) = @_;
+    my ($self, $curve_parts) = @_;
 
     my $private_str = $self->{'private'}->as_bytes();
 
@@ -294,13 +206,81 @@ sub _get_asn1_parts {
         ASN1_PRIVATE(),
         {
             version => 1,
-            privateKey => $private_str,
+            privateKey => $self->_pad_bytes_for_asn1($private_str),
             parameters => $curve_parts,
         },
-        @params,
     );
 }
 
+#Accepts der
+#sub new {
+#    my ($class, $der) = @_;
+#
+#    Crypt::Perl::ToDER::ensure_der($der);
+#
+#    my $asn1 = $class->_asn1();
+#    my $asn1_ec = $asn1->find('ECPrivateKey');
+#
+#    my $struct;
+#    try {
+#        $struct = $asn1_ec->decode($der);
+#    }
+#    catch {
+#        my $ec_err = $_;
+#
+#        my $asn1_pkcs8 = $asn1->find('PrivateKeyInfo');
+#
+#        try {
+#            my $pk8_struct = $asn1_pkcs8->decode($der);
+#
+#            #It still might succeed, even if this is wrong, so don’t die().
+#            if ( $pk8_struct->{'privateKeyAlgorithm'}{'algorithm'} ne $class->OID_ecPublicKey() ) {
+#                warn "Unknown private key algorithm OID: “$pk8_struct->{'privateKeyAlgorithm'}{'algorithm'}”";
+#            }
+#
+#            my $asn1_params = $asn1->find('EcpkParameters');
+#            my $params = $asn1_params->decode($pk8_struct->{'privateKeyAlgorithm'}{'parameters'});
+#
+#            $struct = $asn1_ec->decode($pk8_struct->{'privateKey'});
+#            $struct->{'parameters'} = $params;
+#        }
+#        catch {
+#            die "Failed to decode private key as either ECDSA native ($ec_err) or PKCS8 ($_)";
+#        };
+#    };
+#
+#    my $self = {
+#        version => $struct->{'version'},
+#        private => Crypt::Perl::BigInt->from_bytes($struct->{'privateKey'}),
+#        public => Crypt::Perl::BigInt->from_bytes($struct->{'publicKey'}[0]),
+#
+#        #for parsing
+#        public_bytes_r => \$struct->{'publicKey'}[0],
+#    };
+##print "fieldType [$struct->{'parameters'}{'primeData'}{'fieldType'}]\n";
+#
+#    bless $self, $class;
+#
+#    $self->_add_params( $struct->{'parameters'} );
+#
+#    return $self;
+#}
+
+#could be faster; see JS implementation?
+sub _getBigRandom {
+    my ($limit) = @_;
+
+    my $lim_bytes = length($limit->as_hex()) - 2;
+    $lim_bytes /= 2;
+
+    my $r;
+    do {
+        $r = Crypt::Perl::BigInt->from_hex( Crypt::Perl::RNG::bytes_hex($lim_bytes) );
+    } while $r > $limit;
+
+    return $r;
+}
+
 sub _serialize_sig {
     my ($self, $r, $s) = @_;
 
