@@ -3,16 +3,12 @@ package Crypt::Perl::RSA::KeyBase;
 use strict;
 use warnings;
 
-use parent qw(
-    Class::Accessor::Fast
-    Crypt::Perl::KeyBase
-);
+use parent qw(Class::Accessor::Fast);
 
 use Crypt::Format ();
 use Module::Load ();
 
 use Crypt::Perl::BigInt ();
-use Crypt::Perl::X ();
 
 BEGIN {
     __PACKAGE__->mk_ro_accessors('modulus');
@@ -22,8 +18,6 @@ BEGIN {
     *E = \&publicExponent;
 }
 
-use constant _JWK_THUMBPRINT_JSON_ORDER => qw( e kty n );
-
 sub new {
     my ($class, @args) = @_;
 
@@ -85,37 +79,9 @@ sub to_der {
     return $self->_to_der($self->_ASN1_MACRO());
 }
 
-use constant OID_rsaEncryption => '1.2.840.113549.1.1.1';
-
-sub _to_subject_public_der {
-    my ($self) = @_;
-
-    my $asn1 = $self->_asn1_find('SubjectPublicKeyInfo');
-
-    return $asn1->encode( {
-        algorithm => {
-            algorithm => OID_rsaEncryption(),
-            parameters => Crypt::Perl::ASN1::NULL(),
-        },
-        subjectPublicKey => $self->_to_der('RSAPublicKey'),
-    } );
-}
-
-sub get_struct_for_public_jwk {
-    my ($self) = @_;
-
-    Module::Load::load('MIME::Base64');
-
-    return {
-        kty => 'RSA',
-        n => MIME::Base64::encode_base64url($self->N()->as_bytes()),
-        e => MIME::Base64::encode_base64url($self->E()->as_bytes()),
-    }
-}
-
 #----------------------------------------------------------------------
 
-sub _asn1_find {
+sub _to_der {
     my ($self, $macro) = @_;
 
     Module::Load::load('Crypt::Perl::ASN1');
@@ -124,13 +90,7 @@ sub _asn1_find {
         Crypt::Perl::RSA::Template::get_template('INTEGER'),
     );
 
-    $asn1->find($macro);
-}
-
-sub _to_der {
-    my ($self, $macro) = @_;
-
-    return $self->_asn1_find($macro)->encode( { %$self } );
+    return $asn1->find($macro)->encode( { %$self } );
 }
 
 sub _verify {
@@ -154,15 +114,14 @@ sub _verify {
     if ($scheme eq 'PKCS1_v1_5') {
         my $key_bytes_length = $self->modulus_byte_length();
         if (length($octets) != $key_bytes_length) {
-            my $err = sprintf( "Invalid PKCS1_v1_5 length: %d (should be %d)", length($octets), $key_bytes_length );
-            die Crypt::Perl::X::create('Generic', $err);
+            die sprintf( "Invalid PKCS1_v1_5 length: %d (should be %d)", length($octets), $key_bytes_length );
         }
 
         Module::Load::load('Crypt::Perl::RSA::PKCS1_v1_5');
         return $digest eq Crypt::Perl::RSA::PKCS1_v1_5::decode($octets, $hasher);
     }
 
-    die Crypt::Perl::X::create('Generic', "Unknown signature scheme: “$scheme”");
+    die "Unknown scheme: “$scheme”";
 }
 
 1;
