@@ -32,9 +32,6 @@ Crypt::Perl::RSA::PrivateKey - object representation of an RSA private key
     my $der = $prkey->to_der();
     my $pem = $prkey->to_pem();
 
-    #For use in creating PKCS #10 CSRs and X.509 certificates
-    my $pub_der = $prkey->to_subject_public_der();
-
     my $pbkey = $prkey->get_public_key();
 
     #----------------------------------------------------------------------
@@ -86,7 +83,6 @@ use parent qw(
 use Module::Load ();
 
 use Crypt::Perl::RNG ();
-use Crypt::Perl::X ();
 
 use constant _PEM_HEADER => 'RSA PRIVATE KEY';
 use constant _ASN1_MACRO => 'RSAPrivateKey';
@@ -115,8 +111,6 @@ BEGIN {
     *DQ = \&exponent2;
 
     *QINV = \&coefficient;
-
-    *to_subject_public_der = __PACKAGE__->can('_to_subject_public_der');
 }
 
 sub sign_RS256 {
@@ -140,38 +134,14 @@ sub sign_RS512 {
 sub get_public_key {
     my ($self) = @_;
 
-    require Crypt::Perl::RSA::PublicKey;
+    Module::Load::load('Crypt::Perl::RSA::PublicKey');
 
     return Crypt::Perl::RSA::PublicKey->new( {
         modulus => $self->{'modulus'},
-        publicExponent => $self->{'publicExponent'},
+        exponent => $self->{'publicExponent'},
     } );
 }
 
-sub get_struct_for_private_jwk {
-    my ($self) = @_;
-
-    require MIME::Base64;
-
-    my $jwk = $self->get_struct_for_public_jwk();
-
-    my %augment = qw(
-        d   D
-        p   P
-        q   Q
-        dp  DP
-        dq  DQ
-        qi  QINV
-    );
-
-    for my $k (keys %augment) {
-        my $accessor = $augment{$k};
-        $jwk->{$k} = MIME::Base64::encode_base64url( $self->$accessor()->as_bytes() );
-    }
-
-    return $jwk;
-}
-
 #----------------------------------------------------------------------
 #This function, in tandem with encrypt_raw(), represents the fundamental
 #mathematical truth on which RSA rests.
@@ -190,19 +160,12 @@ sub decrypt_raw {
     my $p = $self->P();
     my $q = $self->Q();
 
-    my $p1 = $p->copy()->bdec();
-    my $q1 = $q->copy()->bdec();
+    my $xp = ($x % $p)->bmodpow( $self->D() % ($p - 1), $p );
+    my $xq = ($x % $q)->bmodpow( $self->D() % ($q - 1), $q );
 
-    my $xp = $x->copy()->bmod($p)->bmodpow( $self->D()->copy()->bmod($p1), $p );
-    my $xq = $x->copy()->bmod($q)->bmodpow( $self->D()->copy()->bmod($q1), $q );
+    $xp += $p while $xp < $xq;
 
-    #$xp->binc($p) while $xp->blt($xq);
-
-    #return ($xq + ((($xp - $xq) * $self->QINV()) % $p) * $q)->as_bytes();
-
-    my $diff = $xp->bsub($xq)->babs()->bmod($p)->bsub($p)->babs();
-
-    $diff->bmul($self->QINV())->bmod($p)->bmuladd($q, $xq)->as_bytes();
+    return ($xq + ((($xp - $xq) * $self->QINV()) % $p) * $q)->as_bytes();
 }
 
 #----------------------------------------------------------------------
@@ -217,7 +180,7 @@ sub _sign {
     my $sig;
 
     if ($scheme eq 'PKCS1_v1_5') {
-        require Crypt::Perl::RSA::PKCS1_v1_5;
+        Module::Load::load('Crypt::Perl::RSA::PKCS1_v1_5');
 
         my $sig_length = $self->modulus_byte_length();
 
@@ -239,7 +202,7 @@ sub _sign {
         substr( $sig, 0, 0 ) = "\0" x ($sig_length - length $sig);
     }
     else {
-        die Crypt::Perl::X::create('Generic', "Unknown RSA signature scheme: “$scheme”");
+        die "Unknown signature scheme: “$scheme”";
     }
 
     return $sig;
@@ -260,26 +223,29 @@ sub _transform {
         $r = Crypt::Perl::BigInt->from_hex(
             Crypt::Perl::RNG::bytes_hex( $key_bytes_length ),
         );
-    } while ($r->bge($self->N())) || ($r->bgcd($self->N())->bne(1));
+    } while ($r >= $self->N()) || ($r->bgcd($self->N()) != 1);
 
-    $x->bmul( $r->copy()->bmodpow($self->E(), $self->N()) )->bmod($self->N());
+    $x *= $r->copy()->bmodpow($self->E(), $self->N());
+    $x %= $self->N();
 
     #calculate xp and xq
-    my $xp = $x->copy()->bmod($self->P())->bmodpow($self->DP(), $self->P());
-    my $xq = $x->copy()->bmod($self->Q())->bmodpow($self->DQ(), $self->Q());
+    my $xp = ($x % $self->P())->bmodpow($self->DP(), $self->P());
+    my $xq = ($x % $self->Q())->bmodpow($self->DQ(), $self->Q());
 
     #xp must be larger than xq to avoid signed bit usage
-    $xp->badd($self->P()) while $xp->blt($xq);
+    while ($xp < $xq) {
+        $xp += $self->P();
+    }
 
-    my $y = $xp->bsub($xq)->bmul($self->QINV())->bmod($self->P());
-
-    #$y *= $self->Q();
-    #$y += $xq;
-    $y->bmuladd( $self->Q(), $xq );
+    my $y = $xp - $xq;
+    $y *= $self->QINV();
+    $y %= $self->P();
+    $y *= $self->Q();
+    $y += $xq;
 
     #remove effect of random for cryptographic blinding
-    $y->bmul( $r->bmodinv($self->N()) );
-    $y->bmod($self->N());
+    $y *= $r->bmodinv($self->N());
+    $y %= $self->N();
 
     return $y;
 }
