@@ -1,8 +1,5 @@
 package Crypt::Perl::ToDER;
 
-use strict;
-use warnings;
-
 use Crypt::Format ();
 
 #Modifies in-place.
