@@ -8,7 +8,7 @@ use parent 'Crypt::Perl::X::Base';
 sub new {
     my ($class, @params) = @_;
 
-    return $class->SUPER::new( "This library has no curve that matches these parameters: [@params]" );
+    return $class->SUPER::new( "This library has no curve that matches these parameters: [@parts]" );
 }
 
 1;
